//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "Ms3FrameListUnit.h"
#include "Ms3ViewerUnit.h"
#include "Ms3Recorder.h"
#include "DataFrame.h"
#include "Log.h"
#include "display\ValueStringFormatter.h"
#include "device\MiniscopeDeviceInterface.h"
#include <algorithm>
#include <time.h>
#include <stdio.h>
#include <assert.h>
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TfrmMs3FrameList *frmMs3FrameList;
//---------------------------------------------------------------------------
__fastcall TfrmMs3FrameList::TfrmMs3FrameList(TComponent* Owner)
	: TForm(Owner)
{
}
//---------------------------------------------------------------------------

void TfrmMs3FrameList::UpdateView(void)
{
	lvFrames->OnChange = NULL;
	lvFrames->Selected = NULL;
	lvFrames->Items->Count = frmMs3Viewer->frame_cnt;
	for (int i=selection.first; i<selection.first + selection.count; i++)
	{
		lvFrames->Items->Item[i]->Selected = true;
	}
	if (selection.count > 0)
	{
		lvFrames->Items->Item[selection.first]->MakeVisible(true);
		lvFrames->Items->Item[selection.first]->Focused = true;
		Ms3Recorder::Instance().GetFrameDesc(selection.first, selection.desc);
	}
	lvFrames->Invalidate();
	lvFrames->OnChange = lvFramesChange;
}

void __fastcall TfrmMs3FrameList::lvFramesData(TObject *Sender, TListItem *Item)
{
	int id = Item->Index;
	struct S_DATAFRAME desc;
	int status = Ms3Recorder::Instance().GetFrameDesc(id, desc);
	if (status)
		return;
	Item->Caption = id + 1;

	char* timeline = ctime( & ( desc.timestamp.time ) );
	timeline[19]=0;
	char buf[64];
	sprintf(buf, "%15s.%03hu", &timeline[4], desc.timestamp.millitm);
	Item->SubItems->Add(buf);

	float fSamplVal = (float)1.0 / desc.fDivX;
	Item->SubItems->Add(ValueStringFormatter::FreqToStr(fSamplVal, 1).c_str());

	Item->SubItems->Add(desc.buffer_size);

	Item->SubItems->Add(DeviceInterface::GetTriggerTypeName((E_TRIGGER_TYPE)desc.trigger_type));

	Item->SubItems->Add(DeviceInterface::GetCouplingTypeName((E_COUPLING_TYPE)desc.coupling));
}
//---------------------------------------------------------------------------


void __fastcall TfrmMs3FrameList::lvFramesChange(TObject *Sender,
	  TListItem *Item, TItemChange Change)
{
	/** \note Behavior:
		Form allows to select several consecutive data frames if some criteria
		are met. Currently only same sampling speed is required, user may decide
		if he wants to see frames on the same page even if data is not continuous.
		If criteria are not met or selected frame is not located next to previously
		selected then all previously selected frames are deselected.
	*/
	
	if (Change != ctState)
		return;
	if (!Item)
	{
		LOG(E_LOG_TRACE, "Item == NULL");
		return;
	}

	TListView *lv = dynamic_cast<TListView*>(Sender);
	assert(lv);

	lv->OnChange = NULL;

	int changed = Item->Index;	// important; Item value changes even when reading listview state

	LOG(E_LOG_TRACE, "Changed item #%02d, currently selected: from %d, %d items", changed, selection.first, selection.count);


	lv->OnSelectItem = NULL;	// clear current handler

	if (!lv->Selected)
	{
		// no item selected on the list
		selection.first = 0;
		selection.count = 0;
		LOG(E_LOG_TRACE, "lv->Selected == NULL");
	}
	else if (lv->SelCount == 1)
	{
		LOG(E_LOG_TRACE, "lv->SelCount == 1");

		int id = lv->Selected->Index;
		int status = Ms3Recorder::Instance().GetFrameDesc(id, selection.desc);
		if (status)
		{
			lv->Selected = NULL;
			selection.first = 0;
			selection.count = 0;
		}
		else
		{
			selection.first = id;
			selection.count = 1;
		}
	}
	else
	{
		int start, end;
		if (selection.first > changed)
		{
			start = changed;
			end = selection.first + selection.count - 1;
		}
		else
		{
			start = selection.first;
			end = std::max(changed, selection.first + selection.count - 1);
		}
		bool bDiscontinuous = false;
		for (int i=start+1; i<=end-1; i++)
		{
			if (lv->Items->Item[i]->Selected == false)
			{
				bDiscontinuous = true;
				break;
			}
		}
		if (!bDiscontinuous)
		{
			if (selection.first > changed)
			{
				start = changed;
				end = selection.first - 1;
			}
			else
			{
				start = selection.first + selection.count;
				end = changed;
			}
			for (int i=start; i<=end; i++)
			{
				struct S_DATAFRAME desc;
				int status = Ms3Recorder::Instance().GetFrameDesc(i, desc);
				if (status)
				{
					bDiscontinuous = true;
					break;
				}
				// check frame parameters consistency
				// currently only same sampling frequency is required
				if (desc.fDivX != selection.desc.fDivX)
				{
					bDiscontinuous = true;
					break;
				}
				if (i == start && i == changed)
				{
                    // update timestamp
					selection.desc = desc;
				}
			}
		}
		if (bDiscontinuous)
		{
			// selected frames are not next to each other - select only newly selected frame
			lv->Selected = NULL;
			lv->Items->Item[changed]->Selected = true;
			struct S_DATAFRAME desc;
			int status = Ms3Recorder::Instance().GetFrameDesc(changed, desc);
			if (status)
			{
				selection.first = 0;
				selection.count = 0;
			}
			else
			{
				selection.first = changed;
				selection.count = 1;
			}
			tmrRefresh->Enabled = true;
			lv->OnChange = lvFramesChange;	// restore previous handler
			return;
		}
		selection.first = lv->Selected->Index;
		selection.count = lv->SelCount;
	}

	tmrRefresh->Enabled = true;
	lv->OnChange = lvFramesChange;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMs3FrameList::tmrRefreshTimer(TObject *Sender)
{
	tmrRefresh->Enabled = false;
	frmMs3Viewer->LoadFrames(selection.first, selection.count);
}
//---------------------------------------------------------------------------

