/** \file Settings.h
*/

#ifndef SettingsH
#define SettingsH
//---------------------------------------------------------------------------
#include <System.hpp>

/** \brief Common (not related to any dll) program settings with Read/Write functions
*/
class Settings
{
public:
	bool Read(AnsiString asFileName);  ///< write settings to ini file
	bool Write(AnsiString asFileName); ///< read settings from ini file
	int iPosX, iPosY;                  ///< main window coordinates
	int iHeight, iWidth;			   ///< main window size
//	bool bAutoScroll;				    ///< auto scroll log
	unsigned int iMaxUiLogLines;	   ///< maximum no. of lines cached with GUI
	bool bAlwaysOnTop;                 ///< window always on top setting
	bool bWindowMaximized;             ///< was window maximized when app was closing?
	unsigned int iDisplayRefreshFreq;	///< display refresh frequency (if applicable)
	AnsiString asInputDeviceDllName;   ///< name of previously selected device interface dll
	AnsiString asPlotPictureExportDir;	///< default directory to save exported plot picture bitmap
	AnsiString asPlotDataExportDir;		///< default directory to save exported plot as JSON file
	struct S_GRAPH
	{
		bool bGridX;
		bool bGridY;
		bool bGridLabels;
	};
	struct S_GRAPH GraphOscilloscope;
	struct S_GRAPH GraphSpectrum;
	struct S_GRAPH GraphRecorder;
	int GraphToolboxTransparency;

	// "Statistics" module settings
	struct S_STATISTICS
	{
		bool bVisible;
		int iPosX, iPosY;
	} Statistics;

	struct S_MS3RECORDING
	{
		int iSizeLimit;		///< recording file size limit [kB]
	} Ms3Recording;

	float fRecorderInterval;
public:
	static const float fRecorderMinInterval;
	static const float fRecorderMaxInterval;
};

#endif
